from datetime import datetime

options = {
    # Available user actions.
    "actions": {
        "setup-os": {
            "title": "Setup OS",
            "description": "Set up the OS: create user accounts and perform OS configuration.",
            "abbreviation": "so"
        },
        "register": {
            "title": "Register CaptureCast Recorder",
            "abbreviation": "rs"
        },
        "install": {
            "title": "Install CaptureCast Platform",
            "description": "Implies --register and --install-drivers; may also imply --setup-os and --install-extras",
            "abbreviation": "i"
        },
        "install-drivers": {
            "title": "Install Drivers Only",
            "abbreviation": "id"
        },
        "install-extras": {
            "title": "Install Extras",
            "description": "Install extra packages (such as web browser and video player)",
            "abbreviation": "ie"
        },
        "check": {
            "title": "Check Installation",
            "description": "Run checks to make sure things installed correctly: drivers, plugins, video devices, etc.",
            "abbreviation": "ch"
        },
        "check-updates": {
            "title": "Check for Updates",
            "description": "Check for available updates to Cattura software.",
            "abbreviation": "cu"
        },
        "uninstall": {
            "title": "Uninstall CaptureCast Platform",
            "abbreviation": "ui"
        },
        "downgrade": {
            "title": "Downgrade CaptureCast Platform",
            "description": "Downgrade CaptureCast Platform to older version. Technically this performs an uninstall and then install. See also --reinstall",
            "abbreviaton": "dg"
        },
        "reinstall": {
            "title": "Reinstall CaptureCast Platform",
            "description": "Reinstall CaptureCast Platform. This performs an uninstall and then reinstall. See also --downgrade",
            "abbreviation": "re"
        },
        "unregister": {
            "title": "Unregister CaptureCast Recorder",
            "description": "Also removes repo configuration. Does not imply --uninstall or --wipe-data.",
            "abbreviation": "ur"
        },
        "wipe-data": {
            "title": "Wipe stored data",
            "description": "Wipes storage directory and database. Does not imply --uninstall or --unregister.",
            "abbreviation": "wd"
        },
        "nuke": {
            "title": "Nuke CaptureCast",
            "description": "Wipe all traces of CaptureCast from the recorder; implies --uninstall, --wipe-data, --unregister",
            "abbreviation": "nu"
        },
        "install-command-center": {
            "title": "Install Cattura Command Center",
            "description": "Installs the Cattura Command Center.",
            "abbreviation": "icc"
        },
        "get-registration-info": {
            "title": "Get registration info",
            "description": "Retrieve registration info (such as license key and expiration)",
            "abbreviation": "gr"
        },
        "create-user": {
            "title": "Create user",
            "description": "Create and configure the cattura-customer user.",
            "abbreviation": "cru"
        },
        "refresh-user": {
            "title": "Refresh user",
            "description": "Remove and re-create customer user with latest configuration",
            "abbreviation": "reu"
        },
        "remove-user": {
            "title": "Remove customer user",
            "description": "Remove the cattura-customer user.",
            "abbreviation": "ru"
        },
        "remount": {
            "title": "Remount root partition",
            "description": "Remount the root partition as writable. Does nothing if the root is not read-only.",
            "abbreviation": "rm"
        }
    },

    # Repository sources.
    # These are shorthands for known repos.
    "repos": {
        "beta": "beta/current",
        "production": "production/current",
        "prod": "production/current"
    },

    # Device drivers such as magewell and datapath
    "drivers": {
        "magewell-old": {"title": "Magewell Generation 1 Drivers", "packages": "cattura-magewell-drivers"},
        "magewell": {"title": "Magewell Pro Capture Drivers", "packages": "cattura-magewell-pro-capture-drivers"},
        "datapath": {"title": "Datapath Drivers", "packages": "cattura-datapath-drivers"}
    },

    # Publishing destinations such as kaltura, youtube, etc
    "publishers": {
        "standard": {"title": "Standard Publishers", "packages": "cattura-standard-publishing-plugins"},
        "kaltura": {"title": "Kaltura", "packages": "cattura-kaltura-plugin"},
        "youtube": {"title": "YouTube", "packages": "cattura-youtube-plugin"},
        "google-drive": {"title": "Google Drive", "packages": "cattura-google-drive-plugin"},
        "opencast": {"title": "Opencast", "packages": "cattura-matterhorn-publishing-plugin"},
        "panopto": {"title": "Panopto", "packages": "cattura-panopto-plugin"},
        "vimeo": {"title": "Vimeo", "packages": "cattura-vimeo-plugin"},
        "sftp": {"title": "SFTP", "packages": "cattura-sftp-plugin"},
        "ftp": {"title": "FTP", "packages": "cattura-ftp-plugin"},
        "storage": {"title": "Local Storage", "packages": "cattura-local-storage-plugin"}
    },

    # User-facing features such as the editor, scheduler, device capture, etc
    "features": {
        # Standard installation profiles. This is what most people will want.
        "standard": {"title": "Standard Features", "packages": "cattura-standard-features"},
        "editing-station": {"title": "Editing Station Profile", "packages": "cattura-editing-station-features"},

        # Standard collections of plugins. This is granular enough for most people.
        "standard-web": {"title": "Standard Web Plugins", "packages": "cattura-standard-web-plugins"},
        "standard-capture": {"title": "Standard Capture Plugins", "packages": "cattura-standard-capture-plugins"},
        "standard-encoding": {"title": "Standard Encoding Plugins", "packages": "cattura-standard-encoding-plugins"},
        "standard-analysis": {"title": "Standard Analysis Plugins", "packages": "cattura-standard-analysis-plugins"},
        "standard-metadata": {"title": "Standard Metadata Plugins", "packages": "cattura-standard-metadata-plugins"},
        "standard-modification": {"title": "Standard Modification Plugins", "packages": "cattura-standard-modification-plugins"},
        "standard-notifiers": {"title": "Standard Notifier Plugins", "packages": "cattura-standard-notifier-plugins"},

        # A-la-carte customization of specific features. This is advanced stuff.
        "editor": {"title": "Editor", "packages": "cattura-media-package-editor-plugin"},
        "scheduler": {"title": "Scheduler", "packages": "cattura-scheduler-plugin"},
        "webhook": {"title": "Webhooks", "packages": "cattura-webhook-plugin"},
        "email": {"title": "Email Notifications", "packages": "cattura-email-notification-plugin"},
        "stitcher": {"title": "Frame Stitcher", "packages": "cattura-frame-stitcher-plugin"},
        "mediainfo": {"title": "MediaInfo", "packages": "cattura-mediainfo-plugin"},
        "twitter": {"title": "Twitter Notifications", "packages": "cattura-twitter-plugin"},
        "live-preview": {"title": "Live Preview", "packages": "cattura-live-preview-plugin"},
        "learning-engine": {"title": "Learning Engine Integration", "packages": "cattura-learning-engine-plugin"},
        "thumbnail-refresher": {"title": "Device Thumbnail Refresher", "packages": "cattura-device-thumbnail-refresh-plugin"},
        "web-interface": {"title": "Web User Interface", "packages": "cattura-web-ui-plugin"},
        "qt-faststart": {"title": "QT Faststart", "packages": "cattura-qt-faststart-plugin"},
        "v4l": {"title": "Video4Linux Capture", "packages": "cattura-v4l-capture-plugin"},
        "alsa": {"title": "ALSA Capture", "packages": "cattura-alsa-capture-plugin"},
        "writer": {"title": "Writer", "packages": "cattura-xuggler-writer-plugin"},
        "rtmp": {"title": "RTMP Streaming", "packages": "cattura-rtmp-streaming-plugin"},
        "importer": {"title": "Media Package Importer", "packages": "cattura-media-package-puller-plugin"},
        "ppm": {"title": "PPM Viewer", "packages": "cattura-audio-level-viewer-plugin"},
        "log-downloader": {"title": "Log Downloader", "packages": "cattura-log-downloader-plugin"},
        "graph-visualizer": {"title": "Graph Visualizer", "packages": "cattura-graph-visualizer-plugin"},
        "ocr": {"title": "OCR", "packages": "cattura-screen-reader-plugin"},
        "scene-change-detector": {"title": "Scene Change Detector", "packages": "cattura-scene-change-detector-plugin"},
        "audio-dumper": {"title": "Audio Dumper", "packages": "cattura-audio-dumper-plugin"},
        "file-upload-service": {"title": "File Upload Service", "packages": "cattura-web-upload-service-plugin"},
        "web-api": {"title": "Web API", "packages": ["cattura-web-api-plugin", "cattura-configuration-api-plugin", "cattura-graph-builder-api-pugin"]}
    },

    # Backend features. These can't be selectable.
    "backend": {
        "nginx": {"title": "Nginx Frontend Server", "packages": "cattura-nginx"}
    },
}

defaults = {
    "features": ["standard"],
    "publishers": ["standard"],
    "backend": [key for key in options["backend"].keys()]
}

class Config (object):
    def __init__ (self):
        self.database = {
            "type": "postgresql",
            "user": "cattura",
            "password": "c@ttur@",
            "name": "cattura"
        }

        # Cattura repo settings
        self.repo = None # user has not chosen a repo
        self.key = None # key will be prompted for, if necessary

        # Features to install
        self.cattura_path = "/var/cattura"
        self.drivers = None #["magewell-old", "magewell", "datapath"]
        self.publishers = None #["standard"]
        self.features = None #[key for key in options["features"].keys() if key.startswith("standard")]
        self.backend = None #[key for key in options["backend"].keys()]

        # Action settings
        self.dry = False
        self.actions = [{"action": "install", "args": []}]
        self.loglevel = "info"
        self.logfile = "installer-{}.log".format(str(datetime.now()).replace(" ", "-"))
        self.json = False
        self.mode = "terminal"
        self.yes = False

        # Backup argument (for install)
        self.backup = None

    def __str__ (self):
        return repr(vars(self))

def from_parsed_args (args):
    args = dict([item for item in vars(args).items() if item[1] is not None])
    config = Config()

    # Repo
    config.repo = args.get("repo")
    config.key = args.get("key", config.key)

    # Database settings
    config.database["type"] = args.get("database_type", config.database["type"])
    config.database["user"] = args.get("database_user", config.database["user"])
    config.database["name"] = args.get("database_name", config.database["name"])
    config.database["password"] = args.get("database_password", config.database["password"])

    # Features to install
    if "features" in args:
        config.features = [key for key in args.get("features") if key in options["features"]]

    if "publishers" in args:
        config.publishers = [key for key in args.get("publishers") if key in options["publishers"]]

    if "drivers" in args:
        config.drivers = [key for key in args.get("drivers") if key in options["drivers"]]

    # Action settings
    config.dry = args.get("dry", config.dry)
    config.loglevel = args.get("loglevel", config.loglevel)
    config.logfile = args.get("logfile", config.logfile)
    config.json = args.get("json", config.json)
    config.mode = args.get("mode", config.mode)
    config.yes = args.get("yes", config.yes)
    config.backup = args.get("restore_from", config.backup)

    # Parse actions
    default_actions = config.actions
    config.actions = []
    for action_name, value in options["actions"].iteritems():
        action_argument_name = action_name.replace("-", "_")
        if not args.get(action_name.replace("-", "_"), None) is None:
            config.actions.append({
                "action": action_name,
                "args": args.get(action_argument_name)
            })
    config.actions = config.actions if config.actions else default_actions

    return config

default = Config()
