import os
import json
import inspect
from subprocess import check_output, CalledProcessError

def is_root ():
    return os.geteuid() == 0

def is_systemd ():
    try:
        return check_output(["which", "systemctl"]).strip() != ""
    except CalledProcessError:
        return False

# http://stackoverflow.com/a/35483750
class ObjectEncoder (json.JSONEncoder):
    def default (self, obj):
        if hasattr(obj, "to_json"):
            return self.default(obj.to_json())
        elif hasattr(obj, "__dict__"):
            d = dict(
                (key, value)
                for key, value in inspect.getmembers(obj)
                if not key.startswith("__")
                and not inspect.isabstract(value)
                and not inspect.isbuiltin(value)
                and not inspect.isfunction(value)
                and not inspect.isgenerator(value)
                and not inspect.isgeneratorfunction(value)
                and not inspect.ismethod(value)
                and not inspect.ismethoddescriptor(value)
                and not inspect.isroutine(value)
            )
            return self.default(d)
        return obj