from subprocess import check_output
from urlparse import urlparse
from datetime import datetime
import calendar

from .cranium import Cranium
from .packages import getReposFromFile, extract_auth

CATTURA_APT_FILE = "/etc/apt/sources.list.d/cattura.list"
LIMELM_DATETIME_FORMAT = "%Y-%m-%d %H:%M:%S"

def get_unit_id ():
    return check_output("hostname").strip()

def get_registration_details ():
    for repo in getReposFromFile(CATTURA_APT_FILE):
        details = extract_auth(repo.url)
        if details[0][0]:
            return details

def discover_cranium (repo_url):
    while True:
        cranium = Cranium(repo_url)

        try:
            cranium.info()
            return cranium
        except Exception:
            if urlparse(repo_url).path == "/":
                return None
            repo_url = repo_url[:repo_url.rindex("/")]

class Registration (object):
    def __init__ (self, data):
        self.licensed = True
        self.key = data["pkey"]["key"]
        self.email = data["pkey"]["email"]
        self.created = _parse_datetime(data["pkey"]["created"])
        self.activated = None
        self.serial = None
        self.valid = data["valid"]
        
        if data.get("activation", None):
            self.activated = _parse_datetime(data["activation"]["date"])
            self.serial = data["activation"]["extra_data"]

        features = data["pkey"]["features"]["feature"]
        self.licensed_to = _find_feature_value(features, "Licensed to")
        self.expiration_date = _parse_datetime(_find_feature_value(features, "Expiration date"))

def _find_feature_value (features, name):
    try:
        return [feature for feature in features if feature["name"] == name][0]["value"]
    except IndexError:
        return None

def _parse_datetime (value):
    if value is None:
        return None
    
    return calendar.timegm(datetime.strptime(value, LIMELM_DATETIME_FORMAT).timetuple()) * 1000