import logging
from datetime import datetime

from ..registration import discover_cranium, get_registration_details, Registration

logger = logging.getLogger(__name__)

def __call__ (config):
    registration_details = get_registration_details()
    if not registration_details:
        logger.info("No authentication details found for this recorder")
        return {"licensed": False}
    
    (authentication, url) = registration_details
    cranium = discover_cranium(url)
    if not cranium:
        logger.warn("Could not detect lemonade stand instance at %s", url)
        return {"licensed": False}
    
    cranium.token = authentication[1]
    user_details = cranium.get_user("me")

    try:
        user_details.update(cranium.get_unit("{}/activation".format(authentication[0])))
    except Exception:
        logger.warn("Failed to get activation details")
        user_details["valid"] = False
    
    registration = Registration(user_details)
    log_registration(registration)
    return registration

def log_registration (registration):
    logger.info("Registration Details:")
    logger.info(" * Licensed to: %s (%s)", registration.licensed_to, registration.email)
    logger.info(" * License key: %s", registration.key)
    logger.info(" * Serial: %s", registration.serial)
    logger.info(" * License valid: %s", registration.valid)
    logger.info(" * Licensed since: %s", format_timestamp(registration.created))
    logger.info(" * Activated on: %s", format_timestamp(registration.activated))
    logger.info(" * Expires on: %s", format_timestamp(registration.expiration_date))

def format_timestamp (timestamp):
    if timestamp is None:
        return "Never"
    
    return datetime.fromtimestamp(timestamp / 1000)