import os
import logging

from ..config import options
from ..context import Context

logger = logging.getLogger(__name__)

def __call__ (config):
    context = Context(config, logger)
    # Install linux headers
    for kernel_version in scan_linux_kernels():
        try:
            logger.info("Installing Kernel Headers for kernel version: {}".format(kernel_version))
            context.install("linux-headers-{}".format(kernel_version))
        except Exception as e:
            logger.error("Unable to install headers for kernel {}: {}".format(kernel_version, e))
    
    # install user-selected drivers
    drivers = [options["drivers"][key] for key in (context.config.drivers if context.config.drivers else get_standard_drivers())]
    for driver in drivers:
        try:
            logger.info("Installing Driver: {}".format(driver["title"]))
            context.install(driver["packages"])
        except Exception as e:
            logger.warn("Failed to install driver {}: {}. Will roll back.".format(driver["title"], e))
            context.remove(driver["packages"])

def get_standard_drivers ():
    return ["datapath", "magewell", "magewell-old"]

def scan_linux_kernels ():
    VMLINUZ = "vmlinuz-"
    return [entry[len(VMLINUZ):] for entry in os.listdir("/boot") if entry.startswith(VMLINUZ)]