import logging

from ..config import options, defaults
from ..context import Context

logger = logging.getLogger(__name__)

def __call__ (config):
    context = Context(config, logger)
    
    # install user-selected features
    features = [options["features"][feature] for feature in (context.config.features if context.config.features else defaults["features"])]
    logger.info("Installing Features: {}".format(", ".join([feature["title"] for feature in features])))
    context.install(*[feature["packages"] for feature in features])

    # install user-selected publishers
    publishers = [options["publishers"][publisher] for publisher in (context.config.publishers if context.config.publishers else defaults["publishers"])]
    logger.info("Installing Publishers: {}".format(", ".join([publisher["title"] for publisher in publishers])))
    context.install(*[publisher["packages"] for publisher in publishers])