import logging

from ..config import options, defaults
from ..context import Context

logger = logging.getLogger(__name__)

CATTURA_PLATFORM_PACKAGE = "cattura-minimal-distribution"

def __call__ (config):
    context = Context(config, logger)
    
    # install Cattura Platform
    logger.info("Installing Cattura Platform")
    context.install(CATTURA_PLATFORM_PACKAGE, dpkg_options=["--force-confdef", "--force-confold"])
    
    # install backend extras
    backend_packages = [options["backend"][key] for key in (context.config.backend if context.config.backend else defaults["backend"])]
    logger.info("Installing Backend Packages: {}".format(", ".join([package["title"] for package in backend_packages])))
    context.install(*[package["packages"] for package in backend_packages])

def get_standard_backend_packages ():
    return [key for key in options["backend"].keys()]