import os
import logging

from ..context import Context

logger = logging.getLogger(__name__)

EXTRA_REPO_APT_FILE = "/etc/apt/sources.list.d/cattura-extra.list"
EXTRA_REPO_URL = "http://repositories.catturavideo.com/extra/"

CHROME_REPO_APT_FILE = "/etc/apt/sources.list.d/google-chrome.list"
CHROME_REPO_KEY = "https://dl-ssl.google.com/linux/linux_signing_key.pub"
CHROME_REPO_LINE = "[arch=amd64] http://dl.google.com/linux/chrome/deb/ stable main"

def __call__ (config):
    context = Context(config, logger)
    needs_update = False
    
    if not os.path.isfile(EXTRA_REPO_APT_FILE):
        logger.info("Adding Cattura extras repository to apt")
        context.write_privileged("deb {} /".format(EXTRA_REPO_URL), EXTRA_REPO_APT_FILE)
        needs_update = True
    else:
        logger.info("Not adding Cattura extras repository since it already exists")
    
    if not os.path.isfile(CHROME_REPO_APT_FILE):
        logger.info("Retrieving Chrome repo signing key")
        context.call_privileged("wget -q -O - {} | apt-key add -".format(CHROME_REPO_KEY), shell=True)
        
        logger.info("Adding Chrome repository to apt")
        context.write_privileged("deb {}".format(CHROME_REPO_LINE), CHROME_REPO_APT_FILE)
        needs_update = True
    else:
        logger.info("Not adding Chrome repository since it already exists")
    
    # update apt-get
    if needs_update:
        context.apt_update()