from subprocess import check_output
import logging

from ..context import Context

logger = logging.getLogger(__name__)

def __call__ (config):
    context = Context(config, logger)

    root_partition = find_root_partition()
    if not (root_partition and root_partition["options"].startswith("ro,")):
        return

    logger.info("Remounting root partition as writable")
    context.call_privileged(["mount", "-o", "remount,rw", root_partition["source"], "/"])

def find_root_partition ():
    try:
        return [parse_line(line) for line in check_output(["findmnt", "--target", "/"]).split("\n")[1:] if line][0]
    except IndexError:
        return None

def parse_line (line):
    while "  " in line:
        line = line.replace("  ", " ")
    (target, source, fstype, options) = line.split(" ")
    return {
        "target": target,
        "source": source,
        "fstype": fstype,
        "options": options
    }
