import logging
import os
from subprocess import PIPE

from ..capturecast import CaptureCast
from ..context import Context
from ..packages import getReposFromFile, getSystemPackages
from ..commands import install

logger = logging.getLogger(__name__)

CATTURA_APT_FILE = "/etc/apt/sources.list.d/cattura.list"

def __call__ (config):
    context = Context(config, logger)
    capturecast = CaptureCast(context)
    
    logger.info("Updating Cattura Platform")
    
    context.apt_update()
    
    logger.info("Fetching list of installed packages")
    installedPackages = [package for package in getSystemPackages() if "installed" in package.status]
    logger.info("{} packages found.".format(len(installedPackages)))
    
    for repo in getReposFromFile(CATTURA_APT_FILE):
        logger.info("Fetching package list from [{}]".format(repo.url))
        catturaPackages = repo.packages()
        
        logger.info("{} packages found.".format(len(catturaPackages)))
        
        logger.info("Finding packages that need to be updated")
        packagesToUpdate = [package for package in installedPackages if len([catturaPackage for catturaPackage in catturaPackages if catturaPackage.package == package.package]) > 0]
        logger.info("{} packages found.".format(len(packagesToUpdate)))
        
        updatePackages(context, packagesToUpdate)
        if capturecast.is_running() and context.ask_yesno("Updates installed. Restart CaptureCast?"):
            capturecast.restart()

def updatePackages (context, packages):
    process_env = os.environ.copy()
    process_env["CATTURA_NO_RESTART"] = "y"
    context.call_privileged(install([package.package for package in packages]), env=process_env, stdin=PIPE)